/*
 * TismTool © Tis Veugen, 2010-2011
 * Simulation, Verification
 *
 * Tsm.cpp
 * version 2
 */
#include <iomanip>
#include <fstream>
#include <boost/thread/locks.hpp>
#include "Tsm.h"
using namespace std;

namespace TsmRTE
{
  bool Tsm::mbLogConsole  = false;
  bool Tsm::mbLogFile     = true;
  int  Tsm::mLogLevel     = 1;
  bool Tsm::mbSeqFile     = true;
  int  Tsm::mSeqClassId   = 0;

  ofstream      Tsm::mLogWriter;
  boost::mutex  Tsm::mLogLockMutex;
  ofstream      Tsm::mSeqWriter;
  boost::mutex  Tsm::mSeqLockMutex;

  void Tsm::Open()
  {
    if ( mbLogFile )
    {
      string s = FreeTismFile( "Log" );
      mLogWriter.open( s.c_str(), ios::out );
      if ( mLogWriter.fail() )
      {
        cout << "ERROR: opening file: " << s;
      }
    }

    if ( mbSeqFile )
    {
      string s = FreeTismFile( "Seq" );
      mSeqWriter.open( s.c_str(), ios::out );
      if ( mSeqWriter.fail() )
      {
        cout << "ERROR: opening file: " << s;
      }
      else
      {
        Seq( "TismTool, Sequence Diagrams" );
      }
    }
  }

  void Tsm::Log( string s )
  {
    boost::unique_lock<boost::mutex> lock( mLogLockMutex );

    if ( mLogWriter )
    {
      mLogWriter << s << endl;
      mLogWriter.flush();
    }

    if ( mbLogConsole )
    {
      cout << s << endl;
    }
  }

  int Tsm::SeqClass( string s )
  {
    int Id;
    {
      boost::unique_lock<boost::mutex> lock( mSeqLockMutex );
      Id = ++mSeqClassId;
    }
    Seq2( "Class," + s + ",", Id );

    return Id;
  }

  void Tsm::Seq( string s )
  {
    boost::unique_lock<boost::mutex> lock( mSeqLockMutex );
    if ( mSeqWriter )
    {
      mSeqWriter << s << endl;
      mSeqWriter.flush();
    }
  }

  void Tsm::Seq2( string s, int n )
  {
    boost::unique_lock<boost::mutex> lock( mSeqLockMutex );
    if ( mSeqWriter )
    {
      mSeqWriter << s << n << endl;
      mSeqWriter.flush();
    }
  }

  void Tsm::Close()
  {
    if ( mLogWriter )
    {
      mLogWriter.close();
    }

    if ( mSeqWriter )
    {
      mSeqWriter.close();
    }
  }

  string Tsm::FreeTismFile( string sName )
  {
    ifstream fi;
    string s;
    int ix = 0;

    for ( ; ; )
    {
      stringstream sb;

      ix++;
      sb.fill( '0' );
      sb << "Tism" << sName << "_" << setw(4) << ix << ".txt";
      s = sb.str();
      fi.open( s.c_str(), ios::in );
      if ( fi.fail() )
      {
        break;
      }
      fi.close();
    }

    return s;
  }
}
