/*
 * TismTool © Tis Veugen, 2010-2011
 * Application, Simulation, Verification
 *
 * TsmSubFsm.cpp
 * version 1
 */
#include "TsmSubFsm.h"
#include "TsmFsm.h"
#include "TsmHandler.h"
#include "TsmHandlerAction.h"
using namespace std;

namespace TsmRTE
{
  void TsmSubFsm::ExecuteTrigger( TsmHandler& oHandler )
  {
    int ProspectSelf = GetProspect( oHandler.Trigger->PortId, oHandler.Trigger->Command );
    oHandler.Prospect |= ProspectSelf;

    oFsm->ExecuteTrigger( oHandler );
    oFsm->ExecuteHandler( oHandler );

    if ( oHandler.Action == TsmHandlerAction::FINAL )
    {
      ExecuteTriggerless( oHandler );
    }
    else
      if ( oHandler.Action == TsmHandlerAction::QUIT )
      {
        ExecuteTriggerless( oHandler );
      }
      else
        if ( ! oHandler.bTaken )
        {
          // Process the trigger itself, if transition available.
          ExecuteTriggerSubFsm( oHandler );
        }
  }

  void TsmSubFsm::Initial( TsmHandler& oHandler )
  {
    oHandler.Action = TsmHandlerAction::ENTER;
    oFsm->ExecuteHandler( oHandler );

    if ( oHandler.Action == TsmHandlerAction::FINAL )
    {
      ExecuteTriggerless( oHandler );
    }

    if ( oHandler.Action == TsmHandlerAction::QUIT )
    {
      ExecuteTriggerless( oHandler );
    }
  }

  void TsmSubFsm::AtExit( TsmHandler& oHandler )
  {
    oFsm->AtExit( oHandler );
    this->AtMyExit( oHandler );
  }

  TsmFsm* TsmSubFsm::SwapFsms( TsmFsm* Target2Source )
  {
    TsmFsm* h = oFsm;
    TsmBase* hp = oFsm->TsmParent;
    oFsm = Target2Source;
    h->TsmParent = Target2Source->TsmParent;
    oFsm->TsmParent = hp;
    return h;
  }

  string TsmSubFsm::TsmSnap()
  {
    return this->TsmName + "+" + oFsm->TsmSnap();
  }

  void TsmSubFsm::TsmCheckVisit()
  {
    oFsm->TsmCheckVisit();
  }

  void TsmSubFsm::ExecuteTriggerSubFsm( TsmHandler& oHandler )
  {
    // Override for all the SubFsm triggers
    oHandler.Action = TsmHandlerAction::NOK;
  }
}
